/*************************************************************
  allgemeine Defines und Macros
  Hardwareeigenschaften / Zuordnung
  
  ------------------------------------------------------------
  
  Copyright (C) 2017 Thomas Lehmann
  
  Version history:
  1.00   20.06.2017  created

***************************************************************/


#ifndef DEF_H
#define DEF_H

#if ARDUINO >= 100
 #include <Arduino.h>
#else
 #include <WProgram.h>
#endif

#define DEBUG_PIN       0   // Debug Ports aktiv
#define DEBUG_GPS       0   // RX Daten ausgeben
#define DEBUG_SCEDULES  0   // Laufzeitmessung
//#define DEBUG_JB_RX   0   // JB Empfang mitschneiden
// !!! HW_TERM benötig scheinbar jede Menge SRAM, läuft nicht mehr bei < 800 Byte Rest !!!
#define HW_TERM         0   // JetiTele COM Port nicht öffnen, ist für Debugausgaben
#define SOFT_TERM       0   // AltSoftSerial als Consolenschnittstelle

#define _SENS_GPS_      1   // 1 = GPS Sensor ist dabei
#define _SENS_ACC_      0   // 1 = G-Force Sensor ist dabei
#define _BMP180_        0   // 1 = BMP180 steht zur Auswahl, benötigt ~2kB an Flash

#if SOFT_TERM == 1
#undef  _SENS_GPS_
#define _SENS_GPS_      1   // 0 = GPS Sensor ist nicht dabei, hängt an der gleichen COM
#endif

#if HW_TERM == 1
#define TermPut           Serial.print
#define TermPutln         Serial.println
#define TermPutTxt(_Txt)  Serial.print(F(_Txt))
#define TermPutBin(_Val)  Serial.print(_Val, BIN)
#define TermPutDez(_Val)  Serial.print(_Val, DEC)
#define TermPutHex(_Val)  Serial.print(_Val, HEX)
#else
#define TermPut
#define TermPutln
#define TermPutTxt
#define TermPutBin
#endif


#define CNT_SENS_TEMP     7
#define CNT_SENS_TEMP_NTC 4
#define CNT_SENS_TEMP_IR  7

// Hardware
// --------
#define PIN_RPM_HALL      2          // D2 INT0   RPM Hallgeber
#define PIN_FUEL          3          // D3 INT1   Fuel Impulsgeber
#define PIN_FUEL_RESET    4          // D4        Fuel Messung zurücksetzen, es wurde getankt (digital)
//#define PIN_GPS_RX      8          // D8 PCINT0 GPS serielle RX (GPS TX) wird von AltSoftSerial verwendet
//#define PIN_GPS_TX      9          // D9 PCINT1 GPS serielle TX (GPS RX)
#define PIN_TEMP_NTC1    A0          // A0 (14)   NTC Tempsensor 1 (analog)
#define PIN_TEMP_NTC2    A1          // A1 (15)   NTC Tempsensor 2 (analog)
#define PIN_TEMP_NTC3    A2          // A2 (16)   NTC Tempsensor 3 (analog)
#define PIN_TEMP_NTC4    A3          // A3 (17)   NTC Tempsensor 4 (analog)

#if DEBUG_PIN == 1
#define PIN_DBG1         10          // für Testausgaben / Messungen ...
#define PIN_DBG2         11
#endif
// I2C Adr 0x5A = IR Temp MLX90614, 1.
// ....
// I2C Adr 0x60 = IR Temp MLX90614, 7.
// I2C Adr 0x76 = Höhe BMP280
// I2C Adr 0x77 = Höhe BMP180
// I2C Adr 0x77 = Höhe MS5611
// I2C Adr 0x53 = Grafity ADXL345 1. Adr.
// I2C Adr 0x1D = Grafity ADXL345 2. Adr.
#define PIN_I2C_SDA   18             // (A4) I2C Data
#define PIN_I2C_SCL   19             // (A5) I2C Clock

#define PIN_LED        LED_BUILTIN   // Pin13 schaltbare LED auf Platine
  #define PIN_LED_ON   HIGH
  #define PIN_LED_OFF  LOW


#define SIG_SAVE2NV   0x01           // Summenwerte im EEProm sichern


#define TIM_BASE    10               // Timer Zeitbasis 10mS

#define OUT_TGL(_PIN) { static int Sts; digitalWrite( _PIN, Sts ); Sts ^= 1; }

#define SoftReset()   { delay(100); asm volatile ("  jmp 0"); }




#define  ARRAYCNT(x)   ( sizeof(x)/sizeof(x[0]) )

// 8 und 16 Bit Variablen (BYTE/SHORT)
#define BIT_TST(val, bitnb)   (boolean) ( (val>>(bitnb)) &1L)
#define BIT_SET(val, bitnb)   ( val |= (1L << (bitnb)) )
#define BIT_CLR(val, bitnb)   ( val &= (~(1L << (bitnb))) )
#define BIT_TGL(val, bitnb)   ( val ^= (1L << (bitnb)) )

// 32 Bit Variablen (LONG)
//#define LBIT_TST(val, bitnb)  (boolean) ( (val>>(bitnb)) &1L)
//#define LBIT_SET(val, bitnb)  ( val |= (1L << (bitnb)) )
//#define LBIT_CLR(val, bitnb)  ( val &= (~(1L << (bitnb))) )
//#define LBIT_TGL(val, bitnb)  ( val ^= (1L << (bitnb)) )




#endif




