/*************************************************************
  G-Force Sensor
  
  Sensor: ADXL 345 (GY-291)
  I2C Adrtesse 53h

  Ausrichtung der Platine:
    ^^ Flugrichtung ^^
          Z--Y
          |
          X

  G-Force +/- 16G
         positiv      negativ
  Pitch  Nase hoch    Nase runter
  Roll   Quer rechts  Quer links
  ------------------------------------------------------------
  
  Copyright (C) 2017 Thomas Lehmann
  
  Version history:
  1.00   17.07.2017  created

***************************************************************/


#include "SensAcc.h"




// ====
// Init
// ====
bool SensorAcc::Init( uint8_t Id, int8_t _CalX, int8_t _CalY, int8_t _CalZ, int8_t _CalPitch, int8_t _CalRoll )
{
  if( Accel )                                     // bereits gestartet ?
    return( true );
  Accel = new Adafruit_ADXL345_Unified();         // standard I2C Adresse
  if( !Accel->begin() )                           // Sensor angeschlossen ?
    return( false );                              // nö
  Accel->setRange( ADXL345_RANGE_16_G );          // G-Force Bereich setzen, wir wolle das maximale ;)
  Accel->setDataRate( ADXL345_DATARATE_400_HZ );  // set measurement data rate

  // Kalibrierungswerte sichern
  CalX     = _CalX;
  CalY     = _CalY;
  CalZ     = _CalZ;
  CalPitch = _CalPitch;
  CalRoll  = _CalRoll;

  TmoRefresh = ACC_TMO_REFRESH + Id;              // 10mS Timer asynchron zu anderen Sensoren bedienen
  return( true );
}


// ==========
// 10mS Timer
// ==========
void SensorAcc::Tim10mS()
{
  // Sensor aktiv ?
  if( --TmoRefresh == 0 )
    {
    TmoRefresh = ACC_TMO_REFRESH;

    int aPitch;
    int aRoll;

    // get a new sensor event
    sensors_event_t Evt; 
    Accel->getEvent(&Evt);

    // results (acceleration is measured in m/s^2)
    TVal.GForceX = (round(Evt.acceleration.x * 1.01972) / 10.0) - (CalX / 10);
    TVal.GForceY = (round(Evt.acceleration.y * 1.01972) / 10.0) - (CalY / 10);
    TVal.GForceZ = (round(Evt.acceleration.z * 1.01972) / 10.0) + (CalZ / 10);
    TVal.GForceX = TVal.GForceX * -1.0;
    TVal.GForceY = TVal.GForceY * -1.0;

    aPitch = atan2(Evt.acceleration.x, Evt.acceleration.z) * (180 / PI) - (CalPitch);
    aRoll  = atan2(Evt.acceleration.y, Evt.acceleration.z) * (180 / PI) - (CalRoll);
    TVal.Pitch = aPitch * -1;
    TVal.Roll  = aRoll * -1;

    #if HW_TERM == 1
    TmoRefresh = 100;
    TermPutTxt("X: "); TermPut(TVal.GForceX);
    TermPutTxt(" Y: "); TermPut(TVal.GForceY);
    TermPutTxt(" Z: "); TermPut(TVal.GForceZ);
    TermPutTxt(" Pitch: "); TermPut(TVal.Pitch);
    TermPutTxt(" Roll: "); TermPut(TVal.Roll);
    TermPutTxt("       \n");
    #endif

/*
    TermPutTxt(" xCal: "); TermPut(CalX);
    TermPutTxt(" yCal: "); TermPut(CalY);
    TermPutTxt(" zCal: "); TermPut(CalZ);
    TermPutTxt(" pCal: "); TermPut(CalPitch);
    TermPutTxt(" rCal: "); TermPut(CalRoll);
    TermPutTxt("\n\n");
*/
    }
}








