/*************************************************************
  Höhen Sensor
  
  Sensor: MS5611 / BMP180 konfigurierbar
  angeschlossen über I2C, A4=SDA A5=SCL
  ------------------------------------------------------------
  
  Copyright (C) 2017 Thomas Lehmann
  
  Version history:
  1.00   19.06.2017  created

***************************************************************/

#ifndef SENS_ALT_H
#define SENS_ALT_H


#include "Def.h"

// Höhensensor MS5611
#include "ms5611.h"
// Höhensensor BMP180
#if _BMP180_  == 1
#include "SFE_BMP180.h"
#endif


class SensorAlt
{
public:
	//SensorAlt();
	
	bool Init( uint8_t Id, uint8_t Typ );
  void Loop();
  void Tim10mS();

  enum
    {
    ALTTYP_MS6511,
    #if _BMP180_  == 1
    ALTTYP_BMP180,
    #endif
    };
  const char *TypTxt[2] = 
    {
    { "MPS5611 (GY-63)" },
    #if _BMP180_  == 1
    { "BMP180  (GY-68)" },
    #endif
    };

  // Telemetriewerte
  struct
    {
    uint32_t AltitudeRelativ;                // relative Höhe ggü Einschaltzeitpunkt, abgerundet auf 1 Meter Genauigkeit, Telemetriewert
    uint16_t TemperatureDez1;                // akt. Temperatur 29.3
    }TVal;


private:

  uint8_t     SensTyp;                       // aktiver Sensor Typ (ALTTYP_XX)
  MS5611     *SensMS5611;                    // Drucksensor MS5611
  #if _BMP180_  == 1
  SFE_BMP180 *SensBMP180;                    // Drucksensor BMP180
  #endif
  double  RefPressure;                       // Referenz Druck beim Einschalten
  uint8_t TmoRefresh;
    #define ALT_TMO_REFRESH  (500/TIM_BASE)  // alle 500mS eine Messung vornehmen

  #if _BMP180_  == 1
  uint8_t StateBmp;
    enum
      {
      BMP_START_TEMP,
      BMP_GET_TEMP,
      BMP_START_ALT,
      BMP_GET_ALT,
      };
  long TmoBMP180;
  #endif
};

#endif




