/*************************************************************
  Benzin Durchfluss Sensor
 
  ------------------------------------------------------------
  
  Copyright (C) 2017 Thomas Lehmann
  
  Version history:
  1.00   01.07.2017  created

***************************************************************/

#ifndef SENS_FUEL_H
#define SENS_FUEL_H


#include "Def.h"

#define FUEL_FLOWCNT_1L   10500               // # Pulse je Liter lt. Hersteller


// Fuelgeber   > INTx, fallende Flanke löst IRQ aus
// Pin2 oder 3 > INT0/1
// ------------------------------------------------
void FlowIRQ( void );                         // ISR Routine muss statisch sein


class SensorFuel
{
  friend void FlowIRQ( void );

public:

  bool Init( uint8_t *Sig, uint8_t ResetAuto, uint16_t CfgVolumeTank, uint16_t CfgFlowCnt1L, uint32_t NV_FlowCntTank, uint32_t NV_FlowCntTot );
 	void Tim10mS( void );
  void ResetFlowCntTank( void )
    {
    NV.FlowCntTank = 0;                       // entnommene Menge zurücksetzen
    FlowCntLast = -1;                         // Refresh Volumenanzeige
    }
  
  // Telemetriewerte
  struct
    {
    float    Flow;                            // akt. Flow in mL
    uint32_t VolumeRemain;                    // berechneter Wert, verbleibende Menge in mL, !! kann auch negativ werden !!
    uint32_t FlowTotDez2;                     // gesamt entnommene Menge in Liter mit 2 deuimal Stellen
    }TVal;

  // Summenwerte für's EEProm
  struct
    {
    volatile uint32_t FlowCntTank;            // aus dem akt. Tank entnommene Menge in Pulsen
    volatile uint32_t FlowCntTot;             // gesamt entnommene Menge in Impulsen
    }NV;



private:

  uint8_t *Signal;
  uint8_t  PinReset;                          // Rücksetz Taster
  uint16_t VolumeTank;                        // Tankvolumen in mL
  uint16_t FlowCnt1L;                         // # Pulse je Liter
  float    uL_OnePulse;                       // uL / Pulse

  uint32_t FlowCntStart;                      // # Impulse bis NV Sicherung angeleiert wird
  uint32_t FlowCntLast = -1;                  // 1. Refresh auch wenn Sensor neu (FlowCntTot=0)


    #define FLOW_AVG_CNT   8
    #define FLOW_AVG_MSK   0x07
    #define FLOW_AVG_SH    3                  // n/8, # shift right für Divison
  volatile uint32_t MesureAvg[FLOW_AVG_CNT];  // die jeweils letzten n Messungen in uS


  uint8_t TmoVolumeRefresh;
    #define TMO_VOL_REFRESH  (1000/TIM_BASE)  // jede Sek. das Restvolumen berechnen
  uint8_t  TmoFlowCalc;
    #define TMO_FLOW_CALC     (500/TIM_BASE)  // alle 500mS die akt. Durchflussmenge berechnen
  uint8_t TmoFlowOff;
    #define TMO_FLOW_OFF      (500/TIM_BASE)  // max. 500mS ohne Flow Impuls

};


#endif



