/*************************************************************
  GPS Sensor
  
  Sensor: NEO 6M
  der NMEA Stream vom Sensor wird über TiniGps++ Library ausgewertet
  http://arduiniana.org/libraries/tinygpsplus/
  ------------------------------------------------------------
  
  Copyright (C) 2017 Thomas Lehmann
  
  Version history:
  1.00   21.06.2017  created

***************************************************************/


#include "SensGps.h"

#if _SENS_GPS_ == 1


// ====
// Init
// ====
bool SensorGps::Init( uint8_t *Sig, uint8_t Id, uint32_t NV_GpsDistTot )
{
  // SensActiv[SensId] = TRUE;             // Sensoren aktivieren
  if( SoftSerial )                         // bereits gestartet ?
    return( true );
  SoftSerial = new AltSoftSerial;          // serielle auf PIN 8 (RX) + 9 (TX) !! IRQ getrieben !!
  SoftSerial->begin( 9600 );
  delay(10);

  Signal = Sig;
  NV.DistTot = NV_GpsDistTot;              // zurückgelegte Strecke gesamt
  
  Tmo = GPS_TMO_REFRESH + Id;              // 10mS Timer asynchron zu anderen Sensoren bedienen
  ChkFailureTmo = TMO_CHK_FAILURE;         // alle n Sek die Funktionalität prüfen
  return( true );

/*
unsigned char cfg_prt[28] = {
  0xB5,0x62,0x06,0x00,0x14,0x00,
  0x01,0x00,0x00,0x00,0xD0,0x08,0x00,0x00,
  0x00,0xE1,0x00,0x00,0x07,0x00,0x03,0x00,  // 57600
  0x00,0x00,0x00,0x00,
  0xDE,0xC9 };
uint8_t Cnt;
  for( Cnt = 0; Cnt < sizeof(cfg_prt); Cnt++ )
    SoftSerial->write(cfg_prt[Cnt]);  //set baud rate to 115200
  SoftSerial->flush();
  delay(10);
  SoftSerial->end();
  SoftSerial->begin(57600);
*/
}




// ==================================================
// zyklischer Aufruf aus Loop()
// seriellen Stream vom GPS Sensor lesen und bewerten
// ==================================================
void SensorGps::Loop()
{
  if( SoftSerial->available() )           // Zeichen empfangen ?
    #if DEBUG_GPS == 0
    if( GpsPlus.encode( SoftSerial->read()) )   // zur Decodierung geben, TRUE == Frame komplett und gültig (CRC)
      FrameCnt++;
    #else
    {
    char c = SoftSerial->read();
    //TermPut(c);
    if( GpsPlus.encode(c) )               // Frame komplett ?
      {
      FrameCnt++;
/*
      TermPutTxt("\n");
      TermPutTxt("Valid: ");
      TermPut( GpsPlus.location.isValid() );
      TermPutTxt(" upd: ");
      TermPut( GpsPlus.location.isUpdated() );
      TermPutTxt(" age: ");
      TermPut( GpsPlus.location.age(), DEC );
      TermPutTxt(" Sats: ");
      TermPutln( GpsPlus.satellites.value() );
*/
      }
    }
    #endif
}



// ==========
// 10mS Timer
// ==========
void SensorGps::Tim10mS()
{
  // auf Fehler/Probleme prüfen
  if( ChkFailureTmo && (--ChkFailureTmo == 0) )
    {
    static uint16_t LastFrameCnt;

    ChkFailureTmo = TMO_CHK_FAILURE;
    if( FrameCnt < LastFrameCnt )         // empfangen wir gültige Daten ?
      Failure = true;
    else
      Failure = false;
    LastFrameCnt = FrameCnt + 20;         // min. 20 Frames sollten es schon sein
    // GpsPlus.charsProcessed()           // empfangen wir Zeichen vom Sensor ?
    // GpsPlus.failedChecksum()           // # Fehlerhafte Frames seit letzter Prüfung
    }

  if( --Tmo == 0 )
    {
    Tmo = GPS_TMO_REFRESH;
    if( !Failure )
      RefreshSensors();                  // zyklsich gelesene Daten ablegen
    }
}


// ==================================
// Refresh aller aktivierten Sensoren
// ==================================
void SensorGps::RefreshSensors()
{
  // IsUpdated() wird gesetzt wenn entspr. Info vom GPS empfangen wurden
  // lesen des entspr. Wertes löscht IsUpdated Flag

  // Empfang OK ? wird Pos zyklisch gemeldet ? Infos frisch ?
  if( !GpsPlus.location.isValid() || !GpsPlus.location.isUpdated() || GpsPlus.location.age() > 2000 )
    return;

  // Position
  TVal.Sats     = GpsPlus.satellites.value();
  TVal.Latitude = GpsPlus.location.lat();
  TVal.Longitude= GpsPlus.location.lng();

  if( !HomePos.Known )                  // Startposition unbekannt ?
    {
    static uint8_t Retrys;
    if( Retrys++ < GPS_TMO_HOMESET )    // lassen wir es mal ein klein wenig einschwingen ...
      return;
    HomePos.Known = true;
    HomePos.Lat = Trip.LastLat = GpsPlus.location.lat();
    HomePos.Lon = Trip.LastLon = GpsPlus.location.lng();
    HomePos.Alt = Trip.LastAlt = GpsPlus.altitude.meters();
//  Trip.Dist = 0;
    }


  // Höhe
  if( GpsPlus.altitude.isUpdated() )
    {
    /*
    static uint8_t Idx;
    static float   Avg[8];
    static float   Sum;
    // Durschnitt von n Messungen
    Sum -= Avg[Idx];
    Avg[Idx] = GpsPlus.altitude.meters();
    Sum += Avg[Idx];
    ++Idx &= 0x07;

    // bei Start Höhe über NN, nach Pos-Update relative Höhe ggü. Startpunkt
    if( (TValAltitude = (Sum/8) - HomePos.Alt) < 0 )
      TValAltitude = 0;                 // wir wollen ja nicht unterirdisch fliegen ;)
    */
    // bei Start Höhe über NN, nach Pos-Update relative Höhe ggü. Startpunkt
    if( (TVal.Altitude = GpsPlus.altitude.meters() - HomePos.Alt) < 0 )
      TVal.Altitude = 0;                // wir wollen ja nicht unterirdisch fliegen ;)
    }

  
  // Geschwindigkeit
  if( GpsPlus.speed.isUpdated() )
    {
    TVal.Speed = GpsPlus.speed.kmph();
    if( TVal.Speed > TVal.SpeedMax )
      TVal.SpeedMax = TVal.Speed;
    }

  // akt. Richtung
  if( GpsPlus.course.isUpdated() )
    TVal.Heading = GpsPlus.course.deg();


  // Entfernung zum Modell vom Startpunkt gesehen
  TVal.DistTo = GpsPlus.distanceBetween( HomePos.Lat, HomePos.Lon, GpsPlus.location.lat(), GpsPlus.location.lng() );
  // Richtung zum Modell vom Startpunkt gesehen
  TVal.CourseTo = GpsPlus.courseTo( HomePos.Lat, HomePos.Lon, GpsPlus.location.lat(), GpsPlus.location.lng() );
  
  
  // gesamt zurückgelegte Strecke
  float Dist = GpsPlus.distanceBetween( Trip.LastLat, Trip.LastLon, GpsPlus.location.lat(), GpsPlus.location.lng() );
  // 3D Distanz
  float AltDiff;
  if( Trip.LastAlt >= TVal.Altitude )
    AltDiff = Trip.LastAlt - TVal.Altitude;
  else
    AltDiff = TVal.Altitude - Trip.LastAlt;
  Dist = sqrt(pow(AltDiff,2) + pow(Dist,2));
  if( Dist > 2 )                       // bewegen wir uns ?
    {
    *Signal |= SIG_SAVE2NV;            // Summenwerte bei Stillstand speichern
    Trip.Dist  += Dist;
    NV.DistTot += Dist;                // Gesamt zurückgelegte Strecke
    Trip.LastLat  = GpsPlus.location.lat();
    Trip.LastLon  = GpsPlus.location.lng();
    Trip.LastAlt  = TVal.Altitude;
    }
  
  // Tele Anzeige
  TVal.DistTrip     = Trip.Dist;          // Anzeige in m
  TVal.DistTotDez2  = NV.DistTot / 10;    // Anzeige in km, 2345m > 23.45 / 10 = 2.34km
}




#if DEBUG_GPS == 2
TermPutln("");
TermPutTxt("GPS: ");

TermPutTxt(" Sats: ");
TermPut(TVal.Sats);
TermPutTxt(" Lat: ");
TermPut(TVal.Latitude);
TermPutTxt(" Lon: ");
TermPut(TVal.Longitude);
TermPutTxt(" age: ");
TermPut(GpsPlus.location.age());

TermPutTxt(" Alt abs: ");
TermPut(GpsPlus.altitude.meters());
TermPutTxt(" Alt rel: ");
TermPut(HomePos.Alt);
TermPutTxt("/");
TermPut(TVal.Altitude);

TermPutTxt(" Speed: ");
TermPut(TVal.Speed);
TermPutTxt("\n");


const float EIFFEL_TOWER_LAT = 48.85826;
const float EIFFEL_TOWER_LNG = 2.294516;
float distanceKm = GpsPlus.distanceBetween( TValLatitude, TValLongitude, EIFFEL_TOWER_LAT, EIFFEL_TOWER_LNG ) / 1000.0;
float CourseTo = GpsPlus.courseTo( TValLatitude, TValLongitude, EIFFEL_TOWER_LAT, EIFFEL_TOWER_LNG );
TermPutTxt("Dist (km) to Eifel Tower: ");
TermPutln( distanceKm );
TermPutTxt("Heading: ");
TermPutln(CourseTo);
TermPutTxt("\n");
#endif



#endif



